/*
 * Copyright (c) 2010-2024. Institut Pasteur.
 *
 * This file is part of Icy.
 * Icy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Icy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Icy. If not, see <https://www.gnu.org/licenses/>.
 */

package plugins.stef.micromanager.block.capture;

import java.util.ArrayList;
import java.util.List;

import org.bioimageanalysis.icy.common.string.StringUtil;
import org.bioimageanalysis.icy.extension.plugin.annotation_.IcyPluginIcon;
import org.bioimageanalysis.icy.extension.plugin.annotation_.IcyPluginName;
import org.jetbrains.annotations.NotNull;
import org.json.JSONObject;
import org.micromanager.utils.ChannelSpec;
import org.micromanager.utils.MDUtils;

import mmcorej.TaggedImage;
import plugins.adufour.blocks.util.VarList;
import plugins.adufour.vars.lang.VarArray;
import plugins.adufour.vars.lang.VarObject;
import plugins.adufour.vars.util.VarException;
import plugins.stef.micromanager.block.AbstractMicroscopeBlock;
import plugins.stef.micromanager.block.lang.VarChannels;
import plugins.stef.micromanager.block.lang.VarMMGroup;
import plugins.tprovoost.Microscopy.MicroManager.MicroManager;
import plugins.tprovoost.Microscopy.MicroManager.tools.StageMover;

/**
 * Snap multi channel image (Micro-Manager)
 * Class in standby until an EzPlug fix
 *
 * @author Stephane Dallongeville
 */
@IcyPluginName("Snap Channel(s)")
@IcyPluginIcon(path = "/mm_blocks.png")
public class MicroscopeSnapChannels extends AbstractMicroscopeBlock {
    VarObject trigger;
    VarMMGroup group;
    VarChannels channels;
    VarArray<TaggedImage> out;

    public MicroscopeSnapChannels() {
        super();

        trigger = new VarObject("Trigger", null);
        group = new VarMMGroup();
        channels = new VarChannels(group);
        out = new VarArray<>("Tagged image(s)", TaggedImage[].class, new TaggedImage[0]);
    }

    @Override
    public void run() {
        // default
        out.setValue(new TaggedImage[0]);

        final ChannelSpec[] specs = channels.getValue();
        // no channel to process
        if (specs.length == 0)
            return;

        try {
            MicroManager.setChannelGroup(group.getValue());
        }
        catch (final Exception e) {
            throw new VarException(group, "Group value is not valid.");
        }

        final List<TaggedImage> result = new ArrayList<>();

        for (int ch = 0; ch < specs.length; ch++) {
            final ChannelSpec cs = specs[ch];

            // channel not used --> skip
            if (!cs.useChannel)
                continue;

            try {
                final String configName = cs.config;

                // set config
                if (!StringUtil.isEmpty(configName))
                    MicroManager.setConfigForGroup(MicroManager.getChannelGroup(), configName, true);
                // set exposure
                MicroManager.setExposure(cs.exposure);
                // doing Z-Stack ? --> apply Z-offset
                if (cs.doZStack && (cs.zOffset != 0d))
                    StageMover.moveZRelative(cs.zOffset, true);

                // do image acquisition

                final List<TaggedImage> images = MicroManager.snapTaggedImage();

                // restore Z position if needed
                if (cs.doZStack && (cs.zOffset != 0d))
                    StageMover.moveZRelative(-cs.zOffset, true);

                // no image ? --> error
                if (images.isEmpty())
                    throw new Exception("Cannot snap image from Micro-Manager !");

                // set metadata
                for (final TaggedImage image : images) {
                    final JSONObject tags = image.tags;

                    MDUtils.setNumChannels(tags, specs.length);
                    MDUtils.setChannelIndex(tags, ch);
                    MDUtils.setChannelColor(tags, cs.color.getRGB());
                }

                // add to result
                result.addAll(images);
            }
            catch (final Throwable t) {
                throw new VarException(out, t.getMessage());
            }
        }

        // set result
        out.setValue(result.toArray(new TaggedImage[0]));
    }

    @Override
    public void declareInput(final @NotNull VarList inputMap) {
        inputMap.add("trigger", trigger);
        inputMap.add("group", group);
        inputMap.add("channels", channels);
    }

    @Override
    public void declareOutput(final @NotNull VarList outputMap) {
        outputMap.add("out", out);
    }
}
