/*
 * Copyright (c) 2010-2024. Institut Pasteur.
 *
 * This file is part of Icy.
 * Icy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Icy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Icy. If not, see <https://www.gnu.org/licenses/>.
 */

package plugins.stef.micromanager.block.lang;

import org.micromanager.dialogs.ChannelTableModel;
import org.micromanager.utils.ChannelSpec;
import org.micromanager.utils.TooltipTextMaker;
import plugins.adufour.vars.gui.VarEditor;
import plugins.adufour.vars.lang.Var;
import plugins.tprovoost.Microscopy.MicroManager.gui.ChannelTable;

import javax.swing.*;
import java.awt.*;

/**
 * Class in standby until an EzPlug fix
 */
public class VarChannelsEditor extends VarEditor<ChannelSpec[]> {
    protected JPanel mainPanel;
    protected ChannelTable table;

    public VarChannelsEditor(final Var<ChannelSpec[]> variable) {
        super(variable);

        // can be done by super constructor
        if (mainPanel == null)
            initialize();

        setEditorEnabled(true);
    }

    @Override
    protected JPanel createEditorComponent() {
        // initialize if needed
        if (mainPanel == null)
            initialize();

        return mainPanel;
    }

    private void initialize() {
        table = new ChannelTable();

        final JScrollPane scroll = new JScrollPane(table);
        scroll.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
        scroll.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
        // better to give it a very small size
        scroll.setMinimumSize(new Dimension(240, 60));
        scroll.setPreferredSize(new Dimension(240, 100));

        final JButton btnAddChannel = new JButton("Add Channel");
        btnAddChannel.setToolTipText("Create new channel for currently selected channel group");
        btnAddChannel.addActionListener(e -> {
            final ChannelTableModel model = (ChannelTableModel) table.getModel();
            model.addNewChannel();
            model.fireTableDataChanged();
        });
        final JButton btnRemoveChannel = new JButton("Remove Channel");
        btnRemoveChannel.setToolTipText("Remove currently selected channel");
        btnRemoveChannel.addActionListener(e -> {
            final int sel = table.getSelectedRow();

            if (sel > -1) {
                final ChannelTableModel model = (ChannelTableModel) table.getModel();
                model.removeChannel(sel);
                model.fireTableDataChanged();
                if (table.getRowCount() > sel)
                    table.setRowSelectionInterval(sel, sel);
            }
        });

        final JButton upButton = new JButton("Up");
        upButton.setToolTipText(TooltipTextMaker.addHTMLBreaksForTooltip("Move currently selected channel up (Channels higher on list are acquired first)"));
        upButton.addActionListener(e -> {
            final int sel = table.getSelectedRow();

            if (sel > -1) {
                final ChannelTableModel model = (ChannelTableModel) table.getModel();
                final int newSel = model.rowUp(sel);
                model.fireTableDataChanged();
                table.setRowSelectionInterval(newSel, newSel);
            }
        });

        final JButton downButton = new JButton("Down");
        downButton.setToolTipText(TooltipTextMaker.addHTMLBreaksForTooltip("Move currently selected channel down (Channels lower on list are acquired later)"));
        downButton.addActionListener(e -> {
            final int sel = table.getSelectedRow();

            if (sel > -1) {
                final ChannelTableModel model = (ChannelTableModel) table.getModel();
                final int newSel = model.rowDown(sel);
                model.fireTableDataChanged();
                table.setRowSelectionInterval(newSel, newSel);
            }
        });

        final JPanel panelButtons = new JPanel();
        panelButtons.setLayout(new BoxLayout(panelButtons, BoxLayout.X_AXIS));
        panelButtons.add(btnAddChannel);
        panelButtons.add(btnRemoveChannel);
        panelButtons.add(upButton);
        panelButtons.add(downButton);

        mainPanel = new JPanel(new BorderLayout());
        mainPanel.add(scroll, BorderLayout.CENTER);
        mainPanel.add(panelButtons, BorderLayout.NORTH);
        mainPanel.validate();
    }

    @Override
    protected void activateListeners() {
        //
    }

    @Override
    protected void deactivateListeners() {
        //
    }

    @Override
    protected void updateInterfaceValue() {
        //
    }

    public void setGroup(final String group) {
        table.setGroup(group);
    }

    @Override
    public Dimension getPreferredSize() {
        if (mainPanel != null)
            return mainPanel.getPreferredSize();

        return new Dimension(400, 140);
    }

    @Override
    public void setComponentToolTipText(final String s) {
        if (mainPanel != null)
            mainPanel.setToolTipText(s);
    }

    @Override
    protected void setEditorEnabled(final boolean enabled) {
        if (mainPanel != null)
            mainPanel.setEnabled(enabled);
    }
}
