/*
 * Copyright (c) 2010-2024. Institut Pasteur.
 *
 * This file is part of Icy.
 * Icy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Icy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Icy. If not, see <https://www.gnu.org/licenses/>.
 */

package plugins.stef.micromanager.block.setting;

import org.bioimageanalysis.icy.extension.plugin.annotation_.IcyPluginIcon;
import org.bioimageanalysis.icy.extension.plugin.annotation_.IcyPluginName;
import org.jetbrains.annotations.NotNull;
import plugins.adufour.blocks.lang.BlockDescriptor;
import plugins.adufour.blocks.lang.WorkFlow;
import plugins.adufour.blocks.util.VarList;
import plugins.adufour.protocols.Protocols;
import plugins.adufour.protocols.gui.MainFrame;
import plugins.adufour.protocols.gui.ProtocolPanel;
import plugins.adufour.protocols.gui.block.BlockPanel;
import plugins.adufour.protocols.gui.block.WorkFlowContainer;
import plugins.adufour.vars.gui.VarEditor;
import plugins.adufour.vars.gui.swing.ComboBox;
import plugins.adufour.vars.lang.Var;
import plugins.adufour.vars.lang.VarBoolean;
import plugins.adufour.vars.lang.VarObject;
import plugins.adufour.vars.util.VarException;
import plugins.adufour.vars.util.VarListener;
import plugins.stef.micromanager.block.AbstractMicroscopeBlock;
import plugins.stef.micromanager.block.lang.VarMMGroup;
import plugins.stef.micromanager.block.lang.VarMMPreset;
import plugins.tprovoost.Microscopy.MicroManager.MicroManager;

import java.util.List;

/**
 * Set a preset for a specified group (Micro-Manager)
 *
 * @author Stephane Dallongeville
 */
@IcyPluginName("Set Micro-Manager Preset")
@IcyPluginIcon(path = "/mm_blocks.png")
public class MicroscopeSetConfig extends AbstractMicroscopeBlock {
    VarObject trigger;
    VarMMGroup group;
    VarMMPreset preset;
    VarBoolean wait;
    VarBoolean done;

    public MicroscopeSetConfig() {
        super();

        trigger = new VarObject("Trigger", null);
        group = new VarMMGroup();
        preset = new VarMMPreset(group);
        wait = new VarBoolean("Wait", true);
        done = new VarBoolean("Done", false);

        // listen group var to refresh the preset GUI which is disconnected from preset var
        group.addListener(new VarListener<>() {
            @Override
            public void valueChanged(final Var<String> source, final String oldValue, final String newValue) {
                refreshPresetsGUI(newValue);
            }

            @Override
            public void referenceChanged(final Var<String> source, final Var<? extends String> oldReference, final Var<? extends String> newReference) {
                //
            }
        });
    }

    @Override
    public void run() {
        done.setValue(Boolean.FALSE);

        try {
            MicroManager.setConfigForGroup(group.getValue(), preset.getValue(), wait.getValue());
            done.setValue(Boolean.TRUE);
        }
        catch (final Throwable t) {
            throw new VarException(group, t.getMessage());
        }
    }

    @Override
    public void declareInput(final @NotNull VarList inputMap) {
        inputMap.add("trigger", trigger);
        inputMap.add("group", group);
        inputMap.add("preset", preset);
        inputMap.add("wait", wait);
    }

    @Override
    public void declareOutput(final VarList outputMap) {
        if (done != null)
            outputMap.add("done", done);
    }

    public void refreshPresetsGUI(final String group) {
        final MainFrame protocols = Protocols.getInstance();
        if (protocols != null) {
            final ProtocolPanel protocolPanel = protocols.getActiveProtocol();
            if (protocolPanel != null) {
                final WorkFlowContainer wfc = protocolPanel.getWorkFlowContainer();
                final WorkFlow wf = protocolPanel.getWorkFlow();

                // get our block descriptor
                final BlockDescriptor bd = wf.getInputOwner(preset);
                if (bd != null) {
                    // get our block panel
                    final BlockPanel bp = wfc.getBlockPanel(bd);
                    if (bp != null) {
                        // get GUI component for preset var
                        final VarEditor<?> editor = bp.getVarEditor(preset);
                        // should be a combo box
                        if (editor instanceof ComboBox<?>) {
                            @SuppressWarnings("unchecked") final ComboBox<String> combo = (ComboBox<String>) editor;
                            final List<String> presets = MicroManager.getConfigs(group);

                            if (!presets.isEmpty())
                                combo.setDefaultValues(presets.toArray(new String[0]), 0, false);
                        }
                    }
                }
            }
        }
    }
}
