/*
 * Copyright (c) 2010-2024. Institut Pasteur.
 *
 * This file is part of Icy.
 * Icy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Icy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Icy. If not, see <https://www.gnu.org/licenses/>.
 */

package plugins.stef.micromanager.block.stage;

import mmcorej.CMMCore;
import org.bioimageanalysis.icy.extension.plugin.annotation_.IcyPluginIcon;
import org.bioimageanalysis.icy.extension.plugin.annotation_.IcyPluginName;
import org.bioimageanalysis.icy.system.thread.ThreadUtil;
import org.micromanager.api.MultiStagePosition;
import plugins.adufour.blocks.util.VarList;
import plugins.adufour.vars.lang.Var;
import plugins.adufour.vars.lang.VarBoolean;
import plugins.adufour.vars.util.VarException;
import plugins.stef.micromanager.block.AbstractMicroscopeBlock;
import plugins.stef.micromanager.block.setting.MicroscopePositions;

/**
 * Set the stage to given position (Micro-Manager).<br>
 * This block should be used in conjunction with {@link MicroscopePositions} block and an indexer
 * so we can move to each position.
 *
 * @author Stephane Dallongeville
 */
@IcyPluginName("Set Micro-Manager Stage Position")
@IcyPluginIcon(path = "/mm_blocks.png")
public class MicroscopeSetStagePosition extends AbstractMicroscopeBlock {
    Var<MultiStagePosition> input;
    VarBoolean waitForStage;
    VarBoolean done;

    public MicroscopeSetStagePosition() {
        super();

        input = new Var<>("Position", MultiStagePosition.class);
        waitForStage = new VarBoolean("Wait stage", true);
        done = new VarBoolean("Done", false);
    }

    @Override
    public void run() {
        done.setValue(Boolean.FALSE);

        final CMMCore core = getMMCore(true);

        if (waitForStage.getValue()) {
            try {
                MultiStagePosition.goToPosition(input.getValue(), core);
                done.setValue(Boolean.TRUE);
            }
            catch (final Throwable t) {
                throw new VarException(input, t.getMessage());
            }
        }
        else {
            // do it in background
            ThreadUtil.bgRun(() -> {
                try {
                    MultiStagePosition.goToPosition(input.getValue(), core);
                }
                catch (final Throwable t) {
                    throw new VarException(input, t.getMessage());
                }
            });

            done.setValue(Boolean.TRUE);
        }
    }

    @Override
    public void declareInput(final VarList inputMap) {
        if (input != null)
            inputMap.add("position", input);
        if (waitForStage != null)
            inputMap.add("waitStage", waitForStage);
    }

    @Override
    public void declareOutput(final VarList outputMap) {
        if (done != null)
            outputMap.add("done", done);
    }
}
