/*
 * Copyright (c) 2010-2024. Institut Pasteur.
 *
 * This file is part of Icy.
 * Icy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Icy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Icy. If not, see <https://www.gnu.org/licenses/>.
 */

package plugins.stef.micromanager.block.stage;

import org.bioimageanalysis.icy.extension.plugin.annotation_.IcyPluginIcon;
import org.bioimageanalysis.icy.extension.plugin.annotation_.IcyPluginName;
import org.jetbrains.annotations.NotNull;
import plugins.adufour.blocks.util.VarList;
import plugins.adufour.vars.lang.VarBoolean;
import plugins.adufour.vars.lang.VarDouble;
import plugins.adufour.vars.lang.VarObject;
import plugins.adufour.vars.util.VarException;
import plugins.stef.micromanager.block.AbstractMicroscopeBlock;
import plugins.tprovoost.Microscopy.MicroManager.tools.StageMover;

import java.awt.geom.Point2D;

/**
 * Set the stage to given XY position (Micro-Manager).
 *
 * @author Stephane Dallongeville
 */
@IcyPluginName("Set Micro-Manager Stage XY")
@IcyPluginIcon(path = "/mm_blocks.png")
public class MicroscopeSetStageXY extends AbstractMicroscopeBlock {
    VarDouble inX;
    VarDouble inY;
    VarObject trigger = new VarObject("Trigger", null);
    VarBoolean wait = new VarBoolean("Wait Stage", true);
    VarBoolean relative = new VarBoolean("Relative", true);
    VarBoolean done = new VarBoolean("Done", false);

    public MicroscopeSetStageXY() {
        super();

        try {
            final Point2D.Double pos = StageMover.getXY();
            inX = new VarDouble("X", pos.x);
            inY = new VarDouble("Y", pos.y);
        }
        catch (final Throwable t) {
            inX = new VarDouble("X", 0d);
            inY = new VarDouble("Y", 0d);
        }
    }

    @Override
    public void run() {
        done.setValue(Boolean.FALSE);

        try {
            if (relative.getValue())
                StageMover.moveXYRelative(inX.getValue(), inY.getValue(), wait.getValue());
            else
                StageMover.moveXYAbsolute(inX.getValue(), inY.getValue(), wait.getValue());

            done.setValue(Boolean.TRUE);
        }
        catch (final Throwable t) {
            throw new VarException(relative, t.getMessage());
        }
    }

    @Override
    public void declareInput(final @NotNull VarList inputMap) {
        inputMap.add("trigger", trigger);
        inputMap.add("x", inX);
        inputMap.add("y", inY);
        inputMap.add("relative", relative);
        inputMap.add("wait", wait);
    }

    @Override
    public void declareOutput(final @NotNull VarList outputMap) {
        outputMap.add("done", done);
    }
}
