/*
 * Copyright (c) 2010-2024. Institut Pasteur.
 *
 * This file is part of Icy.
 * Icy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Icy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Icy. If not, see <https://www.gnu.org/licenses/>.
 */

package plugins.stef.micromanager.block.stage;

import org.bioimageanalysis.icy.extension.plugin.annotation_.IcyPluginIcon;
import org.bioimageanalysis.icy.extension.plugin.annotation_.IcyPluginName;
import org.jetbrains.annotations.NotNull;
import plugins.adufour.blocks.util.VarList;
import plugins.adufour.vars.lang.VarBoolean;
import plugins.adufour.vars.lang.VarDouble;
import plugins.adufour.vars.lang.VarObject;
import plugins.adufour.vars.util.VarException;
import plugins.stef.micromanager.block.AbstractMicroscopeBlock;
import plugins.tprovoost.Microscopy.MicroManager.tools.StageMover;

/**
 * Set the Z stage to given Z position (Micro-Manager).
 *
 * @author Stephane Dallongeville
 */
@IcyPluginName("Set Micro-Manager Stage Z")
@IcyPluginIcon(path = "/mm_blocks.png")
public class MicroscopeSetStageZ extends AbstractMicroscopeBlock {
    VarDouble inZ;
    VarObject trigger = new VarObject("Trigger", null);
    VarBoolean varWait = new VarBoolean("Wait stage", true);
    VarBoolean varRelative = new VarBoolean("Relative", true);
    VarBoolean varDone = new VarBoolean("Done", false);

    public MicroscopeSetStageZ() {
        try {
            final double z = StageMover.getZ();
            inZ = new VarDouble("Z", z);
        }
        catch (final Throwable t) {
            inZ = new VarDouble("Z", 0d);
        }
    }

    @Override
    public void run() {
        varDone.setValue(Boolean.FALSE);
        try {
            if (varRelative.getValue())
                StageMover.moveZRelative(inZ.getValue(), varWait.getValue());
            else
                StageMover.moveZAbsolute(inZ.getValue(), varWait.getValue());

            varDone.setValue(Boolean.TRUE);
        }
        catch (final Throwable t) {
            throw new VarException(varRelative, t.getMessage());
        }
    }

    @Override
    public void declareInput(final @NotNull VarList inputMap) {
        inputMap.add("trigger", trigger);
        inputMap.add("z", inZ);
        inputMap.add("relative", varRelative);
        inputMap.add("wait", varWait);
    }

    @Override
    public void declareOutput(final @NotNull VarList outputMap) {
        outputMap.add("done", varDone);
    }
}
